// ===========================================================================
// CONTENT  : CLASS LoggerService
// AUTHOR   : Manfred Duchrow
// VERSION  : 2.0 - 15/01/2012
// HISTORY  :
//  08/07/2004  mdu  CREATED
//	23/08/2006	mdu		added		-->	changeLogLevel()
//	15/12/2012	mdu		changed	-->	simplified by using varargs
//
// Copyright (c) 2004-2012, by Manfred Duchrow. All rights reserved.
// ===========================================================================
package org.pfsw.tools.beluga.common;

// ===========================================================================
// IMPORTS
// ===========================================================================
import java.text.MessageFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Properties;

import org.pf.logging.Logger;
import org.pf.logging.PrintStreamLogger;

/**
 * Single access point for all logging activities of this application.
 *
 * @author Manfred Duchrow
 * @version 2.0
 */
public class LoggerService
{
  // =========================================================================
  // CONSTANTS
  // =========================================================================

  // =========================================================================
  // CLASS VARIABLES
  // =========================================================================
	private static LoggerService loggerService = new LoggerService() ;

	// =========================================================================
  // INSTANCE VARIABLES
  // =========================================================================
	private Logger logger = null ;

  // =========================================================================
  // PUBLIC CLASS METHODS
  // =========================================================================
	public static LoggerService getLogger()
	{
		return loggerService ;
	} // getLogger()

	// -------------------------------------------------------------------------

  // =========================================================================
  // CONSTRUCTORS
  // =========================================================================
  /**
   * Initialize the new instance with default values.
   */
	private LoggerService()
	{
		super();
		PrintStreamLogger aLogger = new PrintStreamLogger();
		aLogger.setLogLevel(PrintStreamLogger.LEVEL_INFO);
		this.setLogger(aLogger);
	} // LoggerService()

	// =========================================================================
	// PUBLIC INSTANCE METHODS
	// =========================================================================
	/**
	 * Initialize the logger from the given properties settings.
	 * Currently the following properties are supported:
	 * <dl>
	 * <dt>logging.printstream.file</dt>
	 * <dd>The name of a file to which all logging should be redirected</dd>
	 * <dt>logging.level</dt>
	 * <dd>The log level. Must be one of "NONE", "ERROR", "WARNING", "INFO", "DEBUG"</dd>
	 * </dl>
	 **/
	public void initializePrintStreamLogging(Properties properties)
	{
		PrintStreamLogger newLogger;

		newLogger = new PrintStreamLogger();
		newLogger.initialize(properties);
		this.setLogger(newLogger);
	} // initialize()

	// -------------------------------------------------------------------------

	public void logInfo(String id, String msg, Object... params)
	{
		if (this.logger().isLoggingInfos())
		{
			this.logger().logInfo(this.format(id, msg, params));
		}
	} // logInfo()

	// -------------------------------------------------------------------------

	public void logWarning(String id, String msg, Object... params)
	{
		if (this.logger().isLoggingWarnings())
		{
			this.logger().logWarning(this.format(id, msg, params));
		}
	} // logWarning()

	// -------------------------------------------------------------------------

	public void logError(String id, String msg)
	{
		this.logError(id, msg, (Object[])null);
	} // logError()

	// -------------------------------------------------------------------------

	public void logError(String id, String msg, Throwable ex)
	{
		this.logError(id, msg, null, ex);
	} // logError()

	// -------------------------------------------------------------------------

	public void logError(String id, String msg, Object param1, Throwable ex)
	{
		this.logError(id, msg, new Object[] { param1 }, ex);
	} // logError()

	// -------------------------------------------------------------------------

	public void logError(String id, String msg, Object... params)
	{
		if (this.logger().isLoggingErrors())
		{
			this.logger().logError(this.format(id, msg, params));
		}
	} // logError()

	// -------------------------------------------------------------------------

	public void logError(String id, String msg, Object[] params, Throwable ex)
	{
		if (this.logger().isLoggingErrors())
		{
			this.logger().logError(this.format(id, msg, params), ex);
		}
	} // logError()

	// -------------------------------------------------------------------------

	public void logDebug(String msg, Object... params)
	{
		if (this.logger().isLoggingDebugs())
		{
			this.logger().logDebug(this.format("DEBUG", msg, params));
		}
	} // logDebug()

	// -------------------------------------------------------------------------

	public boolean isDebugOn()
	{
		return this.logger().isLoggingDebugs();
	} // isDebugOn()

	// -------------------------------------------------------------------------

	public void changeLogLevel(String newLevel)
	{
		this.logger().setLogLevel(newLevel);
	} // changeLogLevel()

	// -------------------------------------------------------------------------

	// =========================================================================
	// PROTECTED INSTANCE METHODS
	// =========================================================================
	protected String format(String msg, Object... params)
	{
		return MessageFormat.format(msg, params);
	} // format()

	// -------------------------------------------------------------------------

	protected synchronized String format(String id, String msg, Object... params)
	{
		StringBuffer buffer;

		buffer = new StringBuffer(100);
		buffer.append('[');
		buffer.append(this.getTimestamp());
		buffer.append("] <");
		buffer.append(id);
		buffer.append("> ");
		if (params == null)
		{
			buffer.append(msg);
		}
		else
		{
			buffer.append(this.format(msg, params));
		}
		return buffer.toString();
	} // format()

	// -------------------------------------------------------------------------

	protected String getTimestamp()
	{
		SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS");
		return dateFormat.format(new Date());
	} // getTimestamp()

	// -------------------------------------------------------------------------

	protected void setLogger(Logger aLogger)
	{
		logger = aLogger;
	} // setLogger()

	// -------------------------------------------------------------------------

	protected Logger logger()
	{
		return logger;
	} // logger()

	// -------------------------------------------------------------------------

} // class LoggerService
