// ===========================================================================
// CONTENT  : CLASS DirectoryConfiguration
// AUTHOR   : M.Duchrow
// VERSION  : 1.0 - 18/06/2006
// HISTORY  :
//  18/06/2006  mdu  CREATED
//
// Copyright (c) 2006, by M.Duchrow. All rights reserved.
// ===========================================================================
package org.pfsw.tools.beluga.ldap.directory ;

// ===========================================================================
// IMPORTS
// ===========================================================================
import java.util.ArrayList;
import java.util.List;

import org.pf.directory.DistinguishedName;
import org.pf.text.DefaultMatchRuleParser;
import org.pf.text.MatchRule;
import org.pf.text.MatchRuleParseException;
import org.pf.text.StringUtil;
import org.pf.util.CollectionUtil;

/**
 * Contains all information that is necessary to load and start the directory.
 * That includes a rule to recognize container objects, suffixes and the
 * LDIF files containing the data.  
 *
 * @author M.Duchrow
 * @version 1.0
 */
public class DirectoryConfiguration
{
  // =========================================================================
  // CONSTANTS
  // =========================================================================
	protected static final String DEFAULT_CONTAINER_RULE =
		"objectclass{organization,organizationalUnit,dcObject,domain}" ;

  // =========================================================================
  // INSTANCE VARIABLES
  // =========================================================================
  private MatchRule containerRule = null ;
  public MatchRule getContainerRule() { return containerRule ; }
  protected void setContainerRule( MatchRule newValue ) { containerRule = newValue ; }
  
  // List of DirectoryDataFile
  private List definedDataFiles = new ArrayList() ;
  protected List getDefinedDataFiles() { return definedDataFiles ; }
  protected void setDefinedDataFiles( List newValue ) { definedDataFiles = newValue ; }
  
  // List of DistinguishedName
  private List definedSuffixes = new ArrayList() ;
  protected List getDefinedSuffixes() { return definedSuffixes ; }
  protected void setDefinedSuffixes( List newValue ) { definedSuffixes = newValue ; }
  
  // =========================================================================
  // CONSTRUCTORS
  // =========================================================================
  /**
   * Initialize the new instance with default values.
   */
  public DirectoryConfiguration()
  {
    super() ;
    this.setContainerRecognizingRule( DEFAULT_CONTAINER_RULE ) ;
  } // DirectoryConfiguration() 

  // =========================================================================
  // PUBLIC INSTANCE METHODS
  // =========================================================================
  /**
   * Set the rule that will be used to recognize container objects.
   * That is necessary to do proper DN parsing and associating with
   * DIT nodes.
   */
  public void setContainerRecognizingRule( String rule ) 
	{
  	try
		{
			this.setContainerRule( DefaultMatchRuleParser.parseRule( rule ) ) ;
			this.getContainerRule().ignoreCase( true ) ;
			this.getContainerRule().ignoreCaseInNames( true ) ;
		}
		catch ( MatchRuleParseException e )
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
			this.setContainerRecognizingRule( DEFAULT_CONTAINER_RULE ) ;
		}
	} // setContainerRecognizingRule() 
	
	// -------------------------------------------------------------------------
  
  /**
   * Add the given suffix to the internal list of suffixes
   */
  public void addSuffix( String suffix ) 
	{
  	DistinguishedName suffixDN ;
  	
		if ( ! this.str().isNullOrBlank( suffix ) )
		{
			suffixDN = new DistinguishedName( suffix ) ;
			if ( ! this.getDefinedSuffixes().contains( suffixDN ) )
			{
				this.getDefinedSuffixes().add( suffixDN ) ;
			}
		}
	} // addSuffix() 
	
	// -------------------------------------------------------------------------

  /**
   * Add the given data source to the internal list of data files
   */
  public void addDataSource( DirectoryDataFile dataFile ) 
  {
  	if ( dataFile != null )
  	{
  		if ( ! this.getDefinedDataFiles().contains( dataFile ) )
  		{
  			this.getDefinedDataFiles().add( dataFile ) ;
  		}
  	}
  } // addDataSource() 
  
  // -------------------------------------------------------------------------
  
  /**
   * Returns the data files that must be loaded into the directory server.
   */
  public DirectoryDataFile[] getDataFiles()
	{
		return (DirectoryDataFile[])this.coll().toArray( this.getDefinedDataFiles() ) ;
	} // getDataFiles() 
  
  // -------------------------------------------------------------------------
  
  /**
   * Returns the suffixes to be supported by the directory server.
   */
  public DistinguishedName[] getSuffixes() 
	{
		return (DistinguishedName[])this.coll().toArray( this.getDefinedSuffixes() ) ;
	} // getSuffixes() 
	
	// -------------------------------------------------------------------------
  
  // =========================================================================
  // PROTECTED INSTANCE METHODS
  // =========================================================================

  protected CollectionUtil coll() 
	{
		return CollectionUtil.current() ;
	} // coll() 
	
	// -------------------------------------------------------------------------
  
  protected StringUtil str()
	{
		return StringUtil.current();
	} // str() 

	//-------------------------------------------------------------------------
  
} // class DirectoryConfiguration 
