// ===========================================================================
// CONTENT  : CLASS LDIFDirectoryLoader
// AUTHOR   : Manfred Duchrow
// VERSION  : 1.1 - 18/06/2006
// HISTORY  :
//  06/08/2004  mdu  CREATED
//	18/06/2006	mdu		changed	-->	Usage of FileLocator
//
// Copyright (c) 2004-2006, by Manfred Duchrow. All rights reserved.
// ===========================================================================
package org.pfsw.tools.beluga.ldap.directory ;

// ===========================================================================
// IMPORTS
// ===========================================================================
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;

import org.pf.directory.ldap.LDAPDirEntry;
import org.pf.directory.ldap.LDAPDirEntryContainer;
import org.pf.directory.ldap.LDAPUtil;
import org.pf.directory.ldif.LDIFReader;
import org.pf.file.FileLocator;
import org.pf.osf.ObjectContainer;
import org.pf.text.StringPattern;
import org.pf.text.StringUtil;
import org.pfsw.tools.beluga.common.LoggerService;
import org.pfsw.tools.beluga.ldap.server.ContainerNotFoundException;
import org.pfsw.tools.beluga.ldap.server.IsNotContainerException;
import org.pfsw.tools.beluga.ldap.server.LDAPProcessingException;
import org.pfsw.tools.beluga.ldap.server.UndefinedSuffixException;


/**
 * Loads LDIF data from a file into a Directory instance.
 *
 * @author Manfred Duchrow
 * @version 1.1
 */
public class LDIFDirectoryLoader extends LDIFReader
{
  // =========================================================================
  // CONSTANTS
  // =========================================================================
	protected static final StringPattern URL_STR_PATTERN = 
																							StringPattern.create( "*//:*" ) ;
								
	protected static final String ATTR_MODIFYTIMESTAMP = "modifytimestamp" ; 
	protected static final String ATTR_CREATETIMESTAMP = "createtimestamp" ; 

	// =========================================================================
  // INSTANCE VARIABLES
  // =========================================================================
  private LoggerService logger = null ;
  protected LoggerService logger() { return logger ; }
  protected void logger( LoggerService newValue ) { logger = newValue ; }
  
  // =========================================================================
  // CLASS METHODS
  // =========================================================================

  // =========================================================================
  // CONSTRUCTORS
  // =========================================================================
  /**
   * Initialize the new instance with default values.
   */
  public LDIFDirectoryLoader( LoggerService aLogger )
  {
    super() ;
    this.logger( aLogger ) ;
  } // LDIFDirectoryLoader() 

  // -------------------------------------------------------------------------
  
  // =========================================================================
  // PUBLIC INSTANCE METHODS
  // =========================================================================
  /**
   * Load the LDIF data from the file specified by the given URL into the 
   * given directory instance.
   * 
   * @param directory The directory to where to add the data
   * @param fileLocator The file to load the data from 
   */
  public void loadInto( Directory directory, FileLocator fileLocator ) 
	{
  	ObjectContainer objects ;
		
  	objects = this.loadDataFrom( fileLocator ) ;
  	if ( objects != null )
		{
			this.updateDirectory( directory, objects ) ;
			this.logger().logInfo( Directory.I_LOADED_LDIF_URL, 
					"Successfully loaded LDIF data from {0}", fileLocator.getOriginalFileName() ) ;
		}
	} // loadInto() 

	// -------------------------------------------------------------------------

  // =========================================================================
  // PROTECTED INSTANCE METHODS
  // =========================================================================
  protected ObjectContainer loadDataFrom( FileLocator fileLocator ) 
	{
  	InputStream inputStream ;
  	
  	try
		{
  		inputStream = fileLocator.getInputStream() ;
		}
		catch ( IOException ex1 )
		{
			this.logger().logError( Directory.E_OPEN_LDIF, 
																"Could not open {0}", fileLocator, ex1 ) ;
			return null ;
		}		
		
		try
		{
			return this.read( inputStream ) ;
		}
		catch ( IOException e )
		{
			this.logger().logError( Directory.E_LOADING_LDIF, 
					"Error when loading LDIF data from {0}", fileLocator, e ) ;
		}
		return null ;
	} // loadDataFrom() 

	// -------------------------------------------------------------------------

  protected void updateDirectory( Directory directory, ObjectContainer objects ) 
	{
  	LDAPDirEntry entry ;
  	LDAPDirEntry oldEntry ;
  	LDAPDirEntryContainer container ;

  	for( int i = 0 ; i < objects.size(); i++ )
		{
			entry = (LDAPDirEntry)objects.get(i) ;
			entry.setMetaData( directory.getMetaData() ) ;
			try
			{
				oldEntry = directory.findByDN( entry.getDN() ) ;
				if ( oldEntry == null ) // then add the new entry
				{
					this.setMissingTimestampsOnEntry( entry ) ;
					container = directory.findContainerFor( entry.getDN() ) ;
					if ( container == null )
					{
						this.logger().logError( Directory.E_CONTAINER_NOT_FOUND, 
								"Container for {0} not found", entry.getDN() ) ;	
					}
					else
					{
						if ( directory.isContainer( entry ) )
						{
							entry = new LDAPDirEntryContainer( entry ) ;
						}
						container.addChild( entry ) ;
					}
				}
				else  // update the existing entry
				{
					oldEntry.updateFrom( entry ) ;
				}
			}
			catch ( UndefinedSuffixException ex1 )
			{
				this.logger().logError( Directory.E_UNDEFINED_SUFFIX, 
														"Undefined suffix: {0}", ex1.getMessage() ) ;
			}
			catch ( IsNotContainerException ex2 )
			{
				this.logger().logError( Directory.E_IS_NO_CONTAINER, 
						"Entry {0} is no container", ex2.getMessage() ) ;
			}
			catch ( ContainerNotFoundException ex3 )
			{
				this.logger().logError( Directory.E_CONTAINER_NOT_FOUND, 
						"Container not found: {0}", ex3.getMessage() ) ;
			}
			catch ( LDAPProcessingException ex4 )
			{
				this.logger().logError( Directory.E_UPDATE_EXCEPTION, 
						"Exception when updating directory data from {0}", 
						entry.getDN(), ex4 ) ;
			}
		}
	} // updateDirectory() 

	// -------------------------------------------------------------------------

  /**
   * Sets the createtimestamp and modifytimestamp to the current time if
   * these attributes are not yet set.
   */
  protected void setMissingTimestampsOnEntry( LDAPDirEntry entry ) 
	{
  	String timestamp ;
  	
  	timestamp = this.ldap().createTimestamp() ;
    this.setMissingTimestampOnEntry( entry, ATTR_CREATETIMESTAMP, timestamp ) ;
    this.setMissingTimestampOnEntry( entry, ATTR_MODIFYTIMESTAMP, timestamp ) ;
	} // setMissingTimestampsOnEntry() 
	
	// -------------------------------------------------------------------------
  
  protected void setMissingTimestampOnEntry( LDAPDirEntry entry, String attrName, String timestamp ) 
  {
  	String attrValue ;
  	
  	try
		{
			attrValue = entry.getAttributeAsString( attrName ) ;
		}
		catch ( NoSuchFieldException e )
		{
			attrValue = null ;
		}
  	if ( this.str().isNullOrBlank( attrValue ) )
  	{
  		entry.addValue( attrName, timestamp ) ;
  	}
  } // setMissingTimestampOnEntry() 
  
  // -------------------------------------------------------------------------
  
  protected URL convertToURL( String urlStr ) 
	{
  	try
		{
			return new URL( urlStr ) ;
		}
		catch ( MalformedURLException e )
		{
			this.logger().logError( Directory.E_INVALID_URL, 
							"Invalid URL <{0}> for LDIF file", urlStr ) ;
			return null ;
		}
	} // convertToURL() 

	// -------------------------------------------------------------------------
  
  protected boolean isURL( String str ) 
	{
		return URL_STR_PATTERN.matches( str ) ;
	} // isURL() 

	// -------------------------------------------------------------------------
  
  protected LDAPUtil ldap() 
	{
		return LDAPUtil.current() ;
	} // ldap() 
	
	// -------------------------------------------------------------------------
  
  protected StringUtil str()
	{
		return StringUtil.current();
	} // str() 

	//-------------------------------------------------------------------------
  
} // class LDIFDirectoryLoader 
