// ===========================================================================
// CONTENT  : CLASS DelayResponseInterceptor
// AUTHOR   : Manfred Duchrow
// VERSION  : 1.0 - 13/01/2012
// HISTORY  :
//  13/01/2012  mdu  CREATED
//
// Copyright (c) 2012, by MDCS. All rights reserved.
// ===========================================================================
package org.pfsw.tools.beluga.ldap.interceptors ;

// ===========================================================================
// IMPORTS
// ===========================================================================
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicLong;

import org.pfsw.tools.beluga.common.LoggerService;
import org.pfsw.tools.beluga.ldap.messages.ResponseMessage;
import org.pfsw.tools.beluga.ldap.server.IResponseInterceptor;

/**
 * The purpose of this interceptor is to delay all responses for a configurable
 * time (in milliseconds).
 *
 * @author Manfred Duchrow
 * @version 1.0
 */
public class DelayResponseInterceptor implements IResponseInterceptor
{
  // =========================================================================
  // CONSTANTS
  // =========================================================================

  // =========================================================================
  // INSTANCE VARIABLES
  // =========================================================================
  private long delay = 0 ;
  public long getDelay() { return delay ; }
  public void setDelay( long newValue ) { delay = newValue ; }
  
  private int requestInterval = 0 ;
  protected int getRequestInterval() { return requestInterval ; }
  protected void setRequestInterval( int newValue ) { requestInterval = newValue ; }
  
  private int forRequests = 1 ;
  protected int getForRequests() { return forRequests ; }
  protected void setForRequests( int newValue ) { forRequests = newValue ; }
  
  private AtomicLong requestCounter = new AtomicLong(0);
  private AtomicInteger delayedRequests = new AtomicInteger(0);
  
  // =========================================================================
  // CONSTRUCTORS
  // =========================================================================
  /**
   * Initialize the new instance with default values.
   */
  public DelayResponseInterceptor()
  {
    super() ;
  } // DelayResponseInterceptor() 

  // =========================================================================
  // PUBLIC INSTANCE METHODS
  // =========================================================================
  public ResponseMessage handleResponse(ResponseMessage responseMessage)
  {
  	if (this.checkIfDelayIsWanted(this.requestCounter.incrementAndGet()))
		{			
			if (responseMessage.isSearchResponse())
			{
				if (responseMessage.isSearchResultDone())
				{
					this.delayResponse();
				}
			}
			else
			{
				this.delayResponse();
			}
		}
  	return responseMessage;
  } // handleResponse()
  
  // -------------------------------------------------------------------------
  
  @Override
  public String toString()
  {
  	return "DelayResponseInterceptor(delay " + this.getDelay() + " ms for "
  			+ this.getForRequests() + " requests, each " + this.getRequestInterval() 
  			+ " requests)";
  } // toString()

  // =========================================================================
  // PROTECTED INSTANCE METHODS
  // =========================================================================
  protected boolean checkIfDelayIsWanted(long counter) 
	{
  	int delayed;
  	
  	if (this.getDelay() <= 0)
  	{
  		return false;
  	}
		if (this.getRequestInterval() > 0)
		{
			if ((counter % this.getRequestInterval()) == 0)
			{
				delayed = this.delayedRequests.incrementAndGet();
			}
			else
			{
				delayed = this.delayedRequests.get();
				if ((delayed > 0) && (delayed < this.getForRequests()))
				{
					delayed = this.delayedRequests.incrementAndGet();
				}
				else
				{
					this.delayedRequests.set(0);
					return false;
				}
			}
			if (delayed >= this.getForRequests())
			{
				this.delayedRequests.set(0);
			}
		}
		return true;
	} // checkIfDelayIsWanted() 
	
	// -------------------------------------------------------------------------
  
  protected void delayResponse() 
	{
  	try
  	{
  		this.logger().logInfo("DRI0001", "{0} - Delay for {1} ms ({2})", 
  				this.requestCounter.get(), this.getDelay(), Thread.currentThread().getName());
  		Thread.currentThread().sleep(this.getDelay());
  	}
  	catch (InterruptedException ex)
  	{
  		// Use logger
  		System.err.println("DelayResponseInterceptor has been interrupted.");
  	}
	} // delayResponse() 
	
	// -------------------------------------------------------------------------
  
	protected LoggerService logger() 
	{
		return LoggerService.getLogger() ;
	} // logger() 

	// -------------------------------------------------------------------------
	
} // class DelayResponseInterceptor 
