// ===========================================================================
// CONTENT  : CLASS LDAPRequestParser
// AUTHOR   : Manfred Duchrow
// VERSION  : 1.1 - 27/06/2006
// HISTORY  :
//  08/07/2004  mdu  CREATED
//	27/06/2006	mdu		added		-->	parseDeleteRequest()
//
// Copyright (c) 2004-2006, by Manfred Duchrow. All rights reserved.
// ===========================================================================
package org.pfsw.tools.beluga.ldap.protocol ;

// ===========================================================================
// IMPORTS
// ===========================================================================
import java.util.ArrayList;
import java.util.List;

import org.pf.directory.DistinguishedName;
import org.pf.directory.MultiValueAttribute;
import org.pf.directory.ldap.LDAPDirEntry;
import org.pf.util.CollectionUtil;
import org.pfsw.tools.beluga.ldap.messages.AddRequest;
import org.pfsw.tools.beluga.ldap.messages.AttributeModificationDefinition;
import org.pfsw.tools.beluga.ldap.messages.BindRequest;
import org.pfsw.tools.beluga.ldap.messages.DeleteRequest;
import org.pfsw.tools.beluga.ldap.messages.ModifyRequest;
import org.pfsw.tools.beluga.ldap.messages.RequestMessage;
import org.pfsw.tools.beluga.ldap.messages.SearchRequest;
import org.pfsw.tools.beluga.ldap.messages.UnbindRequest;
import org.pfsw.tools.beluga.ldap.protocol.ber.ProtocolDecoder;
import org.pfsw.tools.beluga.ldap.protocol.ber.SequenceInfo;
import org.pfsw.tools.beluga.ldap.protocol.ber.SubstringFilter;


/**
 * This parser is able to interpret LDAP request messages according to
 * RFC-2251 (LDAPv3).
 *
 * @author Manfred Duchrow
 * @version 1.1
 */
public class LDAPRequestParser extends LDAPProtocol
{
  // =========================================================================
  // CONSTANTS
  // =========================================================================
	protected static final int[] SUPPORTED_VERSIONS = { 3 } ;

	protected static final String E_REQUEST_PARSING			= "REQ001" ;
	protected static final String E_PARSING_BIND				= "REQ002" ;
	protected static final String E_PARSING_UNBIND			= "REQ003" ;
	protected static final String E_PARSING_SEARCH			= "REQ004" ;
	protected static final String E_PARSING_DELETE			= "REQ005" ;
	protected static final String E_PARSING_ADD					= "REQ006" ;
	protected static final String E_PARSING_MODIFY			= "REQ007" ;

	protected static final String I_CONN_NAME							= "RPI001" ;

  // =========================================================================
  // INSTANCE VARIABLES
  // =========================================================================

  // =========================================================================
  // CLASS METHODS
  // =========================================================================

  // =========================================================================
  // CONSTRUCTORS
  // =========================================================================
  /**
   * Initialize the new instance with default values.
   */
  public LDAPRequestParser()
  {
    super() ;
  } // LDAPRequestParser()

  // =========================================================================
  // PUBLIC INSTANCE METHODS
  // =========================================================================
  /**
   * Parse the ASN.1 encoded LDAP request and return the data in a Java object
   */
	public RequestMessage parseRequest( byte[] data )
		throws LDAPRequestException
	{
		ProtocolDecoder decoder;
		RequestMessage request ;
		int messageId;
		int requestId;
		int anInt;

		this.logger().logDebug( "******************* Parsing request **************");
		this.logger().logInfo( I_CONN_NAME, this.getConnectionName() ) ;
		if ( this.isDebuggingRawdataOn() )
		{
			this.debugPrintRawData( data ) ;
		}

		decoder = new ProtocolDecoder( data );
		decoder.useUTF8( USE_UTF8 ) ;

		try
		{
			anInt = decoder.parseSequenceTag();
			this.logger().logDebug( "Code={0}", new Integer(anInt) );

			messageId = decoder.parseInteger();
			this.logger().logDebug( "MessageID={0}", new Integer(messageId) );

			requestId = decoder.parseSequenceTag();
			this.logger().logDebug( "Request={0} / {1}", new Integer(requestId) ,
														this.getRequestName( requestId ) );

		}
		catch ( Exception e )
		{
			this.logger().logError( E_REQUEST_PARSING, "Parsing request;", e ) ;
			throw new LDAPRequestParsingException( e.getMessage() ) ;
		}

		request = this.parseRequest( messageId, requestId, decoder ) ;
		request.setRawData( data ) ;
		request.setMessageId( messageId ) ;
		return request ;
	} // parseRequest()

	// -------------------------------------------------------------------------

  // =========================================================================
  // PROTECTED INSTANCE METHODS
  // =========================================================================
	protected RequestMessage parseRequest( int messageId, int requestId,
			                                   ProtocolDecoder decoder )
		throws LDAPRequestException
	{
		switch ( requestId )
		{
			case LDAP_REQ_BIND :
				return this.parseBindRequest( decoder, messageId ) ;
			case LDAP_REQ_SEARCH :
				return this.parseSearchRequest( decoder ) ;
			case LDAP_REQ_UNBIND :
				return this.parseUnbindRequest( decoder ) ;
			case LDAP_REQ_DELETE :
				return this.parseDeleteRequest( decoder ) ;
			case LDAP_REQ_ADD :
				return this.parseAddRequest( decoder ) ;
			case LDAP_REQ_MODIFY :
				return this.parseModifyRequest( decoder ) ;
		}
		throw new LDAPUnknownRequestException( messageId, requestId ) ;
	} // parseRequest()

	// -------------------------------------------------------------------------

	protected BindRequest parseBindRequest( ProtocolDecoder decoder, int msgId )
		throws LDAPRequestException
	{
		BindRequest request ;
		int version;
		String dn ;
		byte[] pwd ;

		request = new BindRequest() ;
		try
		{
			version = decoder.parseInteger() ;
		}
		catch ( Exception ex )
		{
			this.logger().logError( E_PARSING_BIND, "Parsing bind request - LDAP protocol version;", ex ) ;
			throw new LDAPRequestParsingException( ex.getMessage() ) ;
		}
		this.logger().logDebug( "LDAP Version=" + version ) ;
		if ( ! this.isSupportedVersion(version) )
		{
			throw new LDAPVersionNotSupportedException(version, msgId) ;
		}

		try
		{
			dn = decoder.parseString() ; // true = UTF-8, false = ISO-8859-1
			this.logger().logDebug( "dn: {0}", dn ) ;

			pwd = decoder.parsePassword() ;
			this.logger().logDebug( "Password={0}", new String( pwd, CHAR_ENCODING ) ) ;
		}
		catch ( Exception e )
		{
			this.logger().logError( E_PARSING_BIND, "Parsing bind request;", e ) ;
			throw new LDAPRequestParsingException( e.getMessage() ) ;
		}

		request.setDN( dn ) ;
		request.setPassword( pwd ) ;
		return request ;
	} // parseBindRequest()

	// -------------------------------------------------------------------------

	protected UnbindRequest parseUnbindRequest( ProtocolDecoder decoder )
		throws LDAPRequestException
	{
		UnbindRequest request ;

		request = new UnbindRequest() ;
		return request ;
	} // parseUnbindRequest()

	// -------------------------------------------------------------------------

	protected DeleteRequest parseDeleteRequest( ProtocolDecoder decoder )
		throws LDAPRequestException
	{
		DeleteRequest request ;
		String dn ;

		request = new DeleteRequest() ;
		try
		{
			dn = decoder.parseString( decoder.bytesLeft() ) ;
			this.logger().logDebug( "delete request dn: {0}", dn ) ;
			request.setDN( new DistinguishedName( dn ) ) ;
		}
		catch ( Exception e )
		{
			this.logger().logError( E_PARSING_DELETE, "Parsing delete request;", e );
			throw new LDAPRequestParsingException( e.getMessage() );
		}
		return request;
	} // parseDeleteRequest()

	// -------------------------------------------------------------------------

	protected AddRequest parseAddRequest( ProtocolDecoder decoder ) throws LDAPRequestParsingException
	{
		AddRequest request ;
		String dn ;
		LDAPDirEntry entry ;

		request = new AddRequest() ;

		try
		{
			dn = decoder.parseString() ;
			this.logger().logDebug( "add request dn: {0}", dn ) ;
			entry = new LDAPDirEntry( new DistinguishedName( dn ) ) ;
			entry.updateTimestamps() ;
			request.setEntry( entry ) ;
			this.parseAttributeList( decoder, entry ) ;
		}
		catch ( Exception e )
		{
			this.logger().logError( E_PARSING_ADD, "Parsing add request;", e ) ;
			throw new LDAPRequestParsingException( e.getMessage() ) ;
		}
		return request ;
	} // parseAddRequest()

	// -------------------------------------------------------------------------

	protected ModifyRequest parseModifyRequest( ProtocolDecoder decoder ) throws LDAPRequestParsingException
	{
		ModifyRequest request ;
		String dn ;

		request = new ModifyRequest() ;
		try
		{
			dn = decoder.parseString() ;
			this.logger().logDebug( "modify request dn: {0}", dn ) ;
			request.setDN( new DistinguishedName( dn ) ) ;
			this.parseModificationInstructions( decoder, request ) ;
		}
		catch ( Exception e )
		{
			this.logger().logError( E_PARSING_MODIFY, "Parsing modify request;", e ) ;
			throw new LDAPRequestParsingException( e.getMessage() ) ;
		}
		return request ;
	} // parseModifyRequest()

	// -------------------------------------------------------------------------

	protected SearchRequest parseSearchRequest( ProtocolDecoder decoder )
		throws LDAPRequestException
	{
		SearchRequest request ;
		String str ;
		int value ;
		boolean bool ;
		String[] attrNames ;
		LDAPControl[] controls ;

		request = new SearchRequest() ;
		try
		{
			str = decoder.parseString() ;
			this.logger().logDebug( "search base dn: {0}", str ) ;
			request.setBaseDN( str ) ;

			value = decoder.parseEnumeration() ;
			this.logger().logDebug( "search scope: {0}", value ) ;
			request.setScope( value ) ;

			value = decoder.parseEnumeration() ;
			this.logger().logDebug( "deref aliases: {0}", value ) ;
			request.setDerefAliases( value ) ;

			value = decoder.parseInteger() ;
			this.logger().logDebug( "size limit: {0}", value ) ;
			request.setSizeLimit( value ) ;

			value = decoder.parseInteger() ;
			this.logger().logDebug( "time limit: {0}", value ) ;
			request.setTimeLimit( value ) ;

			bool = decoder.parseBoolean() ;
			this.logger().logDebug( "types only: {0}", bool ) ;
			request.setTypesOnly( bool ) ;

			str = this.parseFilter( decoder ) ;
			this.logger().logDebug( "filter: {0}", str ) ;
			request.setFilter( str ) ;

			if ( decoder.hasMoreData() )
			{
				attrNames = this.parseAttributeNames( decoder ) ;
				this.logger().logDebug( "attr names: {0}", this.str().asString( attrNames ) ) ;
				request.setAttributeNames( attrNames ) ;
			}
			if ( decoder.hasMoreData() )
			{
				controls = this.parseControls( decoder ) ;
				request.setControls( controls ) ;
				for (int i = 0; i < controls.length; i++ )
				{
					this.logger().logDebug( "control: {0}", controls[i].toString() ) ;
				}
			}
		}
		catch ( Exception e )
		{
			this.logger().logError( E_PARSING_SEARCH, "Parsing search request;", e ) ;
			throw new LDAPRequestParsingException( e.getMessage() ) ;
		}
		return request ;
	} // parseSearchRequest()

	// -------------------------------------------------------------------------

	protected String parseFilter( ProtocolDecoder decoder )
		throws Exception
	{
		StringBuffer buffer ;

		buffer = new StringBuffer(100) ;
		this.parseFilter( decoder, buffer ) ;
		return buffer.toString() ;
	} // parseFilter()

	// -------------------------------------------------------------------------

	protected void parseFilter( ProtocolDecoder decoder, StringBuffer buffer )
		throws Exception
	{
		SequenceInfo seqInfo ;
		//int level = counter++ ;

		seqInfo = decoder.parseSequence() ;
		this.logger().logDebug( "SEQUENCE: {0}", seqInfo ) ;
		if ( ( seqInfo.getTag() == LBER_SEQUENCE ) || ( seqInfo.getLength() == 0 ) )
		{
			decoder.setPutBackSequence( seqInfo ) ;
			return ;
		}
		//System.err.println( "(" + level ) ;
		buffer.append( "(" ) ;
		switch ( seqInfo.getTag() )
		{
			case LDAP_AND :
				buffer.append( "&" ) ;
				this.parseSetOfFilters( decoder, seqInfo, buffer ) ;
				break;
			case LDAP_OR :
				buffer.append( "|" ) ;
				this.parseSetOfFilters( decoder, seqInfo, buffer ) ;
				break;
			case LDAP_NOT :
				buffer.append( "!" ) ;
				this.parseFilter( decoder, buffer ) ;
				break;
			case LDAP_EQUALITY_MATCH :
				this.parseAttributeValueAssertion( decoder, buffer, "=" ) ;
				break;
			case LDAP_SUBSTRINGS :
				this.parseSubstringFilter( decoder, buffer ) ;
				break;
			case LDAP_PRESENT :
				this.parseAttributePresent( decoder, buffer, seqInfo.getLength() ) ;
				break;
			case LDAP_GREATER_OR_EQUAL :
				this.parseAttributeValueAssertion( decoder, buffer, ">=" ) ;
				break;
			case LDAP_LESS_OR_EQUAL :
				this.parseAttributeValueAssertion( decoder, buffer, "<=" ) ;
				break;
			case LDAP_APPROX_MATCH :
				throw new UnsupportedFilterOperation( "~=" ) ;
			case LDAP_EXTENSIBLE_MATCH :
				throw new UnsupportedFilterOperation( ":=" ) ;
			default:
				throw new InvalidFilterOperation() ;
		}
		//System.err.println( ")" + level ) ;
		buffer.append( ")" ) ;
	} // parseFilter()

	// -------------------------------------------------------------------------

	protected void parseSetOfFilters( ProtocolDecoder decoder,
																	SequenceInfo seqInfo, StringBuffer buffer )
		throws Exception
	{
		while ( decoder.notDoneForSequence( seqInfo ) )
		{
			this.parseFilter( decoder, buffer ) ;
		}
	} // parseSetOfFilters()

	// -------------------------------------------------------------------------

	protected void parseSubstringFilter( ProtocolDecoder decoder,	StringBuffer buffer )
		throws Exception
	{
		SubstringFilter filter ;

		filter = decoder.parseSubstringFilter() ;
		//buffer.append( '(' ) ;
		buffer.append( filter.getName() ) ;
		buffer.append( '=' ) ;
		buffer.append( filter.getPattern() ) ;
		//buffer.append( ')' ) ;
		this.logger().logDebug( "substring filter: {0}", filter ) ;
	} // parseSubstringFilter()

	// -------------------------------------------------------------------------

	protected void parseAttributePresent( ProtocolDecoder decoder,	StringBuffer buffer, int len )
		throws Exception
	{
		//buffer.append( '(' ) ;
		buffer.append( decoder.parseString( len ) ) ;
		buffer.append( "=*" ) ;
		//buffer.append( ')' ) ;

		this.logger().logDebug( "attr present" ) ;
	} // parseAttributePresent()

	// -------------------------------------------------------------------------

	protected void parseAttributeValueAssertion( ProtocolDecoder decoder, StringBuffer buffer, String comparator )
		throws Exception
	{
		String name ;
		String value ;

		name = decoder.parseString(  ) ;
		this.logger().logDebug( "attr name: {0}", name ) ;
		buffer.append( name ) ;
		buffer.append( comparator ) ;

		value = decoder.parseString() ;
		this.logger().logDebug( "attr value: {0}", value ) ;
		buffer.append( value ) ;
	} // parseAttributeValueAssertion()

	// -------------------------------------------------------------------------

	protected String[] parseAttributeNames( ProtocolDecoder decoder )
		throws Exception
	{
		List attrNames ;
		String name ;
		SequenceInfo seqInfo ;

		seqInfo = decoder.parseSequence() ;
		this.logger().logDebug( "SEQUENCE: {0}", seqInfo ) ;

		attrNames = new ArrayList( seqInfo.getLength() ) ;
		while (  decoder.notDoneForSequence( seqInfo ) )
		{
			name = decoder.parseString() ;
			attrNames.add( name ) ;
		}
		return this.str().asStrings( attrNames ) ;
	} // parseAttributeNames()

	// -------------------------------------------------------------------------

	/**
	 * Parse the attribute names and values from the given decoder and add them
	 * to the given entry.
	 */
	protected void parseAttributeList( ProtocolDecoder decoder, LDAPDirEntry entry )
		throws Exception
	{
		SequenceInfo seqInfo ;
		MultiValueAttribute attribute ;

		seqInfo = decoder.parseSequence() ;
		this.logger().logDebug( "SEQUENCE: {0}", seqInfo ) ;
		while ( decoder.notDoneForSequence( seqInfo ) )
		{
			attribute = this.parseAttributeTypeAndValues( decoder ) ;
			entry.addAttribute( attribute ) ;
		}
	} // parseAttributeList()

	// -------------------------------------------------------------------------

	/**
	 * Parse the attribute names and values from the given decoder and add them
	 * to the given entry.
	 */
	protected MultiValueAttribute parseAttributeTypeAndValues( ProtocolDecoder decoder ) throws Exception
	{
		SequenceInfo seqInfo ;
		SequenceInfo setInfo ;
		String attrName ;
		String attrValue ;
		MultiValueAttribute attribute = null ;

		seqInfo = decoder.parseSequence() ;
		this.logger().logDebug( "SEQUENCE: {0}", seqInfo ) ;
		if ( decoder.notDoneForSequence( seqInfo ) )
		{
			attrName = decoder.parseString() ;
			this.logger().logDebug( "attribute name: {0}", attrName ) ;

			attribute = new MultiValueAttribute( attrName ) ;
			setInfo = decoder.parseSequence() ;
			this.logger().logDebug( "SET: {0}", setInfo ) ;
			if ( setInfo.getTag() == LBER_SET )
			{
				while ( decoder.notDoneForSequence( setInfo ) )
				{
					attrValue = decoder.parseString() ;
					attribute.addValue( attrValue ) ;
					this.logger().logDebug( "attribute value: {0}", attrValue ) ;
				}
			}
		}
		return attribute ;
	} // parseAttributeTypeAndValues()

	// -------------------------------------------------------------------------

	protected void parseModificationInstructions( ProtocolDecoder decoder, ModifyRequest request ) throws Exception
	{
		SequenceInfo seqInfo ;

		seqInfo = decoder.parseSequence() ;
		this.logger().logDebug( "SEQUENCE: {0}", seqInfo ) ;
		while ( decoder.notDoneForSequence( seqInfo ) )
		{
			request.addModification( this.parseModificationInstruction( decoder ) ) ;
		}
	} // parseModificationInstructions()

	// -------------------------------------------------------------------------

	protected AttributeModificationDefinition parseModificationInstruction( ProtocolDecoder decoder ) throws Exception
	{
		AttributeModificationDefinition instruction ;
		MultiValueAttribute attribute ;
		SequenceInfo seqInfo ;
		int operation ;

		seqInfo = decoder.parseSequence() ;
		this.logger().logDebug( "SEQUENCE-modification: {0}", seqInfo ) ;
		operation = decoder.parseEnumeration() ;
		this.logger().logDebug( "operation: {0}", this.getModifyOperationName(operation) ) ;
		instruction = new AttributeModificationDefinition( operation ) ;
		attribute = this.parseAttributeTypeAndValues( decoder ) ;
		instruction.setAttribute( attribute ) ;
		return instruction ;
	} // parseModificationInstruction()

	// -------------------------------------------------------------------------

	protected LDAPControl[] parseControls( ProtocolDecoder decoder )
		throws Exception
	{
		SequenceInfo seqInfo ;
		SequenceInfo ctrlSequence ;
		LDAPControl control ;
		List controls ;
		String oid ;

		seqInfo = decoder.parseSequence() ;
		this.logger().logDebug( "SEQUENCE: {0}", seqInfo ) ;

		controls = new ArrayList() ;
		while ( ! decoder.isEndOfSequence( seqInfo ) )
		{
			ctrlSequence = decoder.parseSequence() ;
			this.logger().logDebug( "SEQUENCE: {0}", ctrlSequence ) ;
			oid = decoder.parseString() ;
			control = new LDAPControl( oid ) ;
			if ( ! decoder.isEndOfSequence( ctrlSequence )  )
			{
				control.setCritical( decoder.parseBoolean() ) ;
				if ( ! decoder.isEndOfSequence( ctrlSequence )  )
				{
					control.setValue( decoder.parseString() ) ;
				}
			}
			controls.add( control ) ;
		}
		return (LDAPControl[])this.coll().toArray( controls ) ;
	} // parseControls()

	// -------------------------------------------------------------------------

	protected String getRequestName( int requestId )
	{
		switch ( requestId )
		{
			case LDAP_REQ_BIND :
				return "Bind" ;
			case LDAP_REQ_UNBIND :
				return "Unbind" ;
			case LDAP_REQ_SEARCH :
				return "Search" ;
			case LDAP_REQ_ADD :
				return "Add" ;
			case LDAP_REQ_MODIFY :
				return "Modify" ;
			case LDAP_REQ_DELETE :
				return "Delete" ;
		}
		return "--- unknown ---" ;
	} // getRequestName()

	// -------------------------------------------------------------------------

	protected String getModifyOperationName( int operation )
	{
		switch ( operation )
		{
			case 0 :
				return "add" ;
			case 1 :
				return "delete" ;
			case 2 :
				return "replace" ;
			default :
				return Integer.toString( operation ) + " unknwon" ;
		}
	} // getModifyOperationName()

	// -------------------------------------------------------------------------

	protected boolean isSupportedVersion( int version )
	{
		for (int i = 0; i < SUPPORTED_VERSIONS.length; i++ )
		{
			if ( version == SUPPORTED_VERSIONS[i] )
			{
				return true ;
			}
		}
		return false ;
	} // isSupportedVersion()

	// -------------------------------------------------------------------------

	protected String getConnectionName()
	{
		return Thread.currentThread().getName() ;
	} // getConnectionName()

	// -------------------------------------------------------------------------

	protected CollectionUtil coll()
	{
		return CollectionUtil.current() ;
	} // coll()

	// -------------------------------------------------------------------------

} // class LDAPRequestParser
