// ===========================================================================
// CONTENT  : CLASS ProtocolDecoder
// AUTHOR   : Manfred Duchrow
// VERSION  : 1.0 - 08/08/2004
// HISTORY  :
//  10/07/2004  mdu  CREATED
//
// Copyright (c) 2004, by Manfred Duchrow. All rights reserved.
// ===========================================================================
package org.pfsw.tools.beluga.ldap.protocol.ber;

// ===========================================================================
// IMPORTS
// ===========================================================================
import org.pfsw.tools.beluga.common.LoggerService;

import com.sun.jndi.ldap.BerDecoder;

/**
 * A protocol decoder that helps to get the data from a BER encoded stream
 * to useful objects.
 *
 * @author Manfred Duchrow
 * @version 1.0
 */
public class ProtocolDecoder
{
  // =========================================================================
  // CONSTANTS
  // =========================================================================
	protected static final int PASSWORD_TAG			= 128 ; 
	
	
  protected static final int WILDCARD_INITIAL							= 130 ; 
  protected static final int WILDCARD_ANY									= 129 ;
  protected static final int WILDCARD_FINAL								= 128 ;
	
  // =========================================================================
  // INSTANCE VARIABLES
  // =========================================================================
  private BerDecoder berDecoder = null ;
  protected BerDecoder getBerDecoder() { return berDecoder ; }
  protected void setBerDecoder( BerDecoder newValue ) { berDecoder = newValue ; }
  
  private boolean useUTF8 = true ;
  public boolean useUTF8() { return useUTF8 ; }
  public void useUTF8( boolean newValue ) { useUTF8 = newValue ; }
  
  private SequenceInfo putBackSequence = null ;
  public SequenceInfo getPutBackSequence() { return putBackSequence ; }
  public void setPutBackSequence( SequenceInfo newValue ) { putBackSequence = newValue ; }
  // =========================================================================
  // CLASS METHODS
  // =========================================================================

	// =========================================================================
	// CONSTRUCTORS
	// =========================================================================
	/**
	 * Initialize the new instance with default values.
	 */
	public ProtocolDecoder(byte[] data)
	{
		super();
		this.setBerDecoder(new BerDecoder(data, 0, data.length));
	} // ProtocolDecoder() 

	// -------------------------------------------------------------------------

	// =========================================================================
	// PUBLIC INSTANCE METHODS
	// =========================================================================
	/**
	 * Returns the next bytes as SEQUENCE information
	 */
	public SequenceInfo parseSequence() throws Exception
	{
		SequenceInfo sequence;
		int tag;
		int[] len = new int[1];

		if (this.hasPutBackSequence())
		{
			sequence = this.getPutBackSequence();
			this.setPutBackSequence(null);
			return sequence;
		}

		tag = this.getBerDecoder().parseSeq(len);
		return new SequenceInfo(tag, len[0], this.getBerDecoder().getParsePosition());
	} // parseSequence() 

	// -------------------------------------------------------------------------

	/**
	 * Returns only the next byte as SEQUENCE tag
	 */
	public int parseSequenceTag() throws Exception
	{
		SequenceInfo sequence;

		sequence = this.parseSequence();
		return sequence.getTag();
	} // parseSequenceTag() 

	// -------------------------------------------------------------------------

	/**
	 * Returns the next integer from the underlying data
	 */
	public int parseInteger() throws Exception
	{
		return this.getBerDecoder().parseInt();
	} // parseInteger() 

	// -------------------------------------------------------------------------

	/**
	 * Returns the next string from the underlying data
	 */
	public String parseString() throws Exception
	{
		return this.parseString(this.useUTF8());
	} // parseString() 

	// -------------------------------------------------------------------------

	/**
	 * Returns the next length bytes as string from the underlying data
	 */
	public String parseString(int length) throws Exception
	{
		byte[] bytes;

		bytes = this.nextBytes(length);
		return new String(bytes, this.useUTF8() ? "UTF-8" : "ISO-8859-1");
	} // parseString() 

	// -------------------------------------------------------------------------

	/**
	 * Returns the password as byte array from the next data in the stream
	 */
	public byte[] parsePassword() throws Exception
	{
		return this.getBerDecoder().parseOctetString(PASSWORD_TAG, null);
	} // parsePassword() 

	// -------------------------------------------------------------------------

	/**
	 * Returns the next data as enumeration tag
	 */
	public int parseEnumeration() throws Exception
	{
		return this.getBerDecoder().parseEnumeration();
	} // parseEnumeration() 

	// -------------------------------------------------------------------------

	/**
	 * Returns next data from the stream as boolean 
	 */
	public boolean parseBoolean() throws Exception
	{
		return this.getBerDecoder().parseBoolean();
	} // parseBoolean() 

	// -------------------------------------------------------------------------

	/**
	 * Returns the next byte ;
	 */
	public int parseByte() throws Exception
	{
		return this.getBerDecoder().parseByte();
	} // parseByte() 

	// -------------------------------------------------------------------------

	public SubstringFilter parseSubstringFilter() throws Exception
	{
		SubstringFilter filter;
		byte[] subdata;
		ProtocolDecoder subDecoder;
		StringBuffer buffer;
		SequenceInfo seqInfo;
		String str;
		boolean wildcardAtEnd = false;

		filter = new SubstringFilter();
		filter.setName(this.parseString());

		seqInfo = this.parseSequence();
		this.logger().logDebug("Substr SeqInfo: {0}", seqInfo);

		subdata = this.nextBytes(seqInfo.getLength());
		subDecoder = new ProtocolDecoder(subdata);

		buffer = new StringBuffer(40);
		while (subDecoder.hasMoreData())
		{
			seqInfo = subDecoder.parseSequence();
			this.logger().logDebug("Substr seq: {0}", seqInfo);
			str = subDecoder.parseString(seqInfo.getLength());
			this.logger().logDebug("Substr: {0}", str);

			switch (seqInfo.getTag())
			{
				case WILDCARD_INITIAL :
					if (!wildcardAtEnd)
						buffer.append('*');
					buffer.append(str);
					wildcardAtEnd = false;
					break;
				case WILDCARD_ANY :
					if (!wildcardAtEnd)
						buffer.append('*');
					buffer.append(str);
					buffer.append('*');
					wildcardAtEnd = true;
					break;
				case WILDCARD_FINAL :
					buffer.append(str);
					buffer.append('*');
					wildcardAtEnd = true;
					break;
			}
		}
		filter.setPattern(buffer.toString());
		return filter;
	} // parseSubstringFilter() 

	// -------------------------------------------------------------------------

	/**
	 * Returns the number of bytes left in the decoder to parse
	 */
	public int bytesLeft()
	{
		return this.getBerDecoder().bytesLeft();
	} // bytesLeft() 

	// -------------------------------------------------------------------------

	/**
	 * Returns true if ther is more data available in the decoder
	 */
	public boolean hasMoreData()
	{
		return (this.getBerDecoder().bytesLeft() > 0);
	} // hasMoreData() 

	// -------------------------------------------------------------------------

	/**
	 * Returns true if the decoder has a sequence that was put back for further
	 * parsing.
	 */
	public boolean hasPutBackSequence()
	{
		return this.getPutBackSequence() != null;
	} // hasPutBackSequence() 

	// -------------------------------------------------------------------------

	/**
	 * Returns true if this decoder has reached the parse position which is the
	 * end of the given sequence info.
	 */
	public boolean isEndOfSequence(SequenceInfo seqInfo)
	{
		return (this.getBerDecoder().getParsePosition() >= seqInfo.getEndPos());
	} // isEndOfSequence() 

	// -------------------------------------------------------------------------

	/**
	 * Returns true if this decoder has NOT yet reached the parse position 
	 * which is the end of the given sequence info and if it still has data.
	 */
	public boolean notDoneForSequence(SequenceInfo seqInfo)
	{
		return ((!this.isEndOfSequence(seqInfo)) && (this.hasMoreData()));
	} // notDoneForSequence() 

	// -------------------------------------------------------------------------

	// =========================================================================
	// PROTECTED INSTANCE METHODS
	// =========================================================================
	/**
	 * Returns the next string from the underlying data
	 */
	protected String parseString(boolean isUTF8) throws Exception
	{
		return this.getBerDecoder().parseString(isUTF8);
	} // parseString() 

	// -------------------------------------------------------------------------

	protected byte[] nextBytes(int len) throws Exception
	{
		byte[] subdata;

		subdata = new byte[len];
		for (int i = 0; i < subdata.length; i++)
		{
			subdata[i] = (byte)this.getBerDecoder().parseByte();
		}
		return subdata;
	} // nextBytes() 

	// -------------------------------------------------------------------------

	protected LoggerService logger()
	{
		return LoggerService.getLogger();
	} // logger() 

	// -------------------------------------------------------------------------

} // class ProtocolDecoder 
